 
package xyz.veronie.bgg.ui.handlers;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;

import xyz.veronie.bgg.result.BggApi;
import xyz.veronie.bgg.result.Thing;
import xyz.veronie.bgg.result.ThingProvider;
import xyz.veronie.bgg.types.EventConstants;
import xyz.veronie.bgg.ui.dialogs.SaveGameListDialog;

public class ImportResultTxtHandler {
	
	private Shell shell;


	@Execute
	public void execute(Shell shell, ThingProvider thingProvider, BggApi bggApi, IEventBroker eventBroker) {
		this.shell = shell;
		
		FileDialog dialog = new FileDialog(shell, SWT.OPEN);
		dialog.setText("Select a result.txt from bgg1tool");
		String[] exfilters = { "*.txt" };
		dialog.setFilterExtensions(exfilters);
		String resultPath = dialog.open();
		
		if(resultPath != null && !resultPath.isEmpty()) {
			List<Integer> thingIds = parseResultTxtIds(resultPath);
			if(thingIds != null && !thingIds.isEmpty()) {
				SaveGameListDialog saveDialog = new SaveGameListDialog(shell);
				saveDialog.open();
	
				int returnCode = saveDialog.getReturnCode();
				if(returnCode == Dialog.OK) {
					String listName = saveDialog.getEntryText();
					try {
						eventBroker.send(EventConstants.TOPIC_STATUS, "Fetching entries from BGG...");
						List<Thing> things = bggApi.getThings(thingIds);
						thingProvider.storeThings(things, listName);
						eventBroker.send(EventConstants.TOPIC_RESULT_CHANGED, listName);
						eventBroker.send(EventConstants.TOPIC_STATUS, "Fetched " + Integer.toString(things.size()) + " things.");
					} catch (SQLException e) {
						MessageBox msgErrorBox = new MessageBox(shell, SWT.ICON_ERROR | SWT.OK);
						msgErrorBox.setMessage("Could not save imported result.txt as game list.");
						msgErrorBox.open();
						e.printStackTrace();
					}
				}
			}
		}
	}
	
	
	private List<Integer> parseResultTxtIds(String resultPath) {
		FileReader input = null;
		String myLine = null;
		int lineNo = 0;
		int errNo = 0;
		try {
			input = new FileReader(resultPath);
			
			BufferedReader bufferedReader = new BufferedReader(input);
			
		    List<Integer> ids = new ArrayList<Integer>();
			while ( (myLine = bufferedReader.readLine()) != null)
			{   				
			    String[] tokens = myLine.split(",");
			    if(tokens.length > 1) {
			    	if(tokens[0].equals("id")) continue; // header line
			    	try {
			    		ids.add(Integer.parseInt(tokens[0]));
			    		lineNo++;
			    	}
			    	catch (NumberFormatException e) {
			    		errNo++;
			    		System.out.println("WARN: Could not parse id from line " + Integer.toString(lineNo) 
			    			+ ", line starts with '" + tokens[0] + "'");
			    	}
			    }
			}
			
			bufferedReader.close();
			
			if(ids != null) {
				MessageBox msgBox = new MessageBox(shell, SWT.ICON_INFORMATION | SWT.OK);
				StringBuilder msg = new StringBuilder();
				msg.append("Parsing successful. Found ").append(Integer.toString(lineNo)).append(" things.");
				if(errNo > 0) {
					msg.append("Skipped ").append(Integer.toString(errNo)).append(" line(s).\n\r");
				} else {
					msg.append("\n\r");
				}
				msg.append("Next, you will be asked to save the imported list as a new game list.");
				msgBox.setMessage(msg.toString());
				msgBox.open();
			}
			
			System.out.println("TRACE: " + ids);
			return ids;
			
		} catch (FileNotFoundException e) {
			MessageBox msgBox = new MessageBox(shell, SWT.ICON_ERROR | SWT.OK);
			msgBox.setMessage("Could not open file '" + resultPath + "'.");
			msgBox.open();
			e.printStackTrace();
		} catch (IOException e) {
			MessageBox msgBox = new MessageBox(shell, SWT.ICON_ERROR | SWT.OK);
			String msg = "Error while parsing '" + resultPath + "'.\r\n" +
					"It must have comma separated lines starting with an integer number.\r\n";
			msgBox.setMessage(msg);
			msgBox.open();
			e.printStackTrace();
		}

		
		return null;
	}


	@CanExecute
	public boolean canExecute() {
		
		return true;
	}
		
}